const mariadb = require('mariadb');
const {log, constants, helpers} = require('utils-nxg-cg');
const {messages_databases} = constants;

/**
 *
 * @param properties
 * @returns {Promise<Object>}
 */
const createConnection = async (properties = {}) => {
    try {
        return mariadb.createPool({
            //change to one minute
            connectTimeout: 60000,
            host: properties.host,
            user: properties.user,
            port: properties.port,
            password: properties.password,
            database: properties.database,
            logger: {
                error: (err) => log.error(err)
            },
            //change to 16 mb
            maxAllowedPacket: 16777216,
        });
    } catch (e) {
        throw Error(e);
    }
};

/**
 *
 * @param properties
 * @returns {Promise<array|object>}
 */
const process = async (properties) => {
    let connection;
    try {
        connection = await createConnection(properties);
        connection = await connection.getConnection();
        let {content, limit, query} = properties;
        let result;
        await connection.beginTransaction();
        //if content is not empty or null
        if (content) {
            log.debug(messages_databases.START_BATCH);
            //check if values is an array []
            if (Array.isArray(content) && content.length > 0) {
                if (content.length > limit)
                    throw Error(`${messages_databases.ERROR_BATCH} ${limit}`);
                const arrays = Array.isArray(content[0]);
                //if arrays then batch without namedPlaceholders
                if (arrays)
                    result = await connection.batch(query, content);
                else {
                    //if first element is an object execute namedPlaceholders
                    if (helpers.isObjectValid(content[0]))
                        result = await connection.batch({sql: query, namedPlaceholders: true}, content);
                    else
                        result = await connection.batch(query, content);
                }
            }
            //check if values is an object {}
            else if (helpers.isObjectValid(content)) {
                result = await connection.query({namedPlaceholders: true, sql: query}, content);
            } else {
                result = await connection.query(query);
            }
        } else {
            log.debug(messages_databases.START_QUERY);
            result = await connection.query(query);
        }
        await connection.commit();
        await connection.release();
        const _result = processResult(result);
        if (helpers.isObjectValid(_result))
            return _result;
        else
            throw Error(constants.ERROR_JSON_FORMAT);
    } catch (e) {
        if (connection) {
            log.error(messages_databases.TRANSACTION_ERROR);
            await connection.rollback();
            await connection.release();
        }
        throw Error(e.toString());
    }
};

/**
 * Method for process a clean output of result mariadb or mysql database
 * @param result
 * @returns {{affectedRows, insertId: number}|{length}|*}
 */
const processResult = (result) => {
    if (Array.isArray(result)) {
        if (!result.length) return result;
        const arrays = Array.isArray(result[0]);
        if (arrays) {
            //this if is to validate is the first array is an array, if then return only first data, apply for mysql response
            if (result[0].length === 1)
                return result[0][0];
            return result[0];
        }
        if (result.length === 1)
            return result[0];
        return result;
    } else if (Object.prototype.toString.call(result) && Object.keys(result).length > 0) {
        const {affectedRows, insertId} = result;
        if (insertId)
            return {affectedRows, insertId: Number(insertId)};
        return {affectedRows};
    }
    return result;
}


module.exports = process;
